# vismap.rb - layer/scene visibility map
# copyright 2009, Martin Rinehart

require 'sketchup'

=begin
Key structure to understand get/setVisibles is the "vis" ("V"s and "I"s). This is a string of "V"s and "I"s, such as "VVIVIIVVI...". "V" == "V"isible; "I" == "I"nvisible. There is one "V" or "I" for each layer/scene intersection. 

If there are 5 scenes, the first five letters in the string are the "V/I" settings of the five scenes in the first layer; the second five letters are the five scenes in the second layer, and so on. Another way to look at it is that these are the checkboxes--"V" if checked, "I" if not--in the map, read as a Western language: left-to-right and top-to-bottom.
=end

class Model

	def initialize()
		puts
		puts 'in Model.initialize()'
		@model = Sketchup.active_model
		puts '@model = ' + @model.to_s
		@layers = @model.layers
		puts '@layers = ' + @layers.to_s
		@scenes = @model.pages
		puts '@scenes = ' + @scenes.to_s
	end

	def layers()
		puts 'in Model.layers() returning ' + @layers.to_s
		return @layers
	end

	def layer_names()
		puts 'in Model.layer_names()'
		names = []
		@layers.each { |lr| names.push(lr.name) }
		return names
	end

	def scenes
		puts 'in Model.scenes'
		return @scenes
	end

	def scene_names()
		puts 'in Model.scene_names()'
		names = []
		@scenes.each { |s| names.push(s.name) }
		return names
	end

    # return "VIVV...", one "V"isbl or "I"nvisbl for each scene in each layer
    def getVisibles()
		puts 'in Model.getVisibles'
    	answer = 'V' * (@scenes.count*@layers.length)
    	i = 0
    	@scenes.each do |s|
			puts 'in Model.getVisibles, working on scene ' + s.name
			s.layers.each do |lr|
				puts 'in Model.getVisibles, working on layer ' + lr.name
				loc = locate( lr.name, layer_names() ) * @scenes.count
				loc += locate( s.name, scene_names() )
				answer[loc..loc] = "I"
			end
			i += 1
    	end
    	return answer

    end # getVisibles()

    # JSON to send to webdialog
    def getJson()
		puts
		puts 'in Model.getJson()'
    	ret = '{ '
		begin
			puts 'in Model.getJson adding layers'
			ret += 'layers:' + namesJson( self.layer_names )
			puts 'in Model.getJson adding scenes'
			ret += ', scenes:' + namesJson( self.scene_names )
			puts 'in Model.getJson adding vis'
			vis = self.getVisibles()
		rescue
			puts 'in Model.getJson, rescue block!---------------------------'
			vis = 'Error: ' + $!
			puts vis
		end
    	ret += ", vis:'" + vis + "'"
    	ret += ' }'
    	return ret

    end # getJson()

	# given "VI" string (see getVisibles) sets visibilities
	def setVisibles( vis )

		sn = 0 # scene number
		@scenes.each do |s|

			ln = 0 # layer number
			@layers.each do |lr|

				vn = ln*@scenes.count + sn # vis number
				v_or_i = vis[vn..vn]

				s.set_visibility( @layers[ln], v_or_i == 'V' )
				ln += 1

			end # layers.each

			sn += 1

		end # scenes.each

	end # of setVisibles()

end # of class Model

def reloadModel()
	$vismap_model = Model.new
end

def launchWebDialog

	puts
	puts 'In launchWebDialog, calling reloadModel()'

	# reloadModel()
	wd = UI::WebDialog.new( "Layer/Scene Visibility Map", false, 
			'vismap_v1.0', 400, 300, 100, 500, true )

    wd.add_action_callback( "refresh" ) do |js_wd, msg|
		puts
		puts 'in action callback, refresh, js_wd = ' + js_wd.to_s
		puts 'in refresh, calling reloadModel()'
		reloadModel()
		puts 'in refresh, calling getJson()'
    	json = $vismap_model.getJson
		puts 'in refresh, json = ' + json
    	script = 'rubyReturned( "' + json + '" );'
		puts 'in refresh,  script = ' + script
    	js_wd.execute_script( script )
    end

    wd.add_action_callback( "newVis" ) do |js_wd, msg|
		reloadModel()
		nlrs = $vismap_model.layers().length
		npgs = $vismap_model.scenes().count
    	if ( (nlrs*npgs) == (msg.length) )
    		$vismap_model.setVisibles( msg )
    	else 
    		UI.messagebox "Wrong number of layers and/or scenes \n" +
    			"Click \"Get Data from Model\" and try again."
    	end
    end
	
	pathname = Sketchup.find_support_file( 'vismap.html', 'Plugins/vismap/' )
    wd.set_file( pathname  )
    wd.show()

end # launchWebDialog()

# true if any member of array matches item
def is_in( item, array )
	return locate( item, array ) > -1
end

# index of item in array (-1 == not found)
def locate( item, array )
	i = 0
	array.each do |a|
		return i if (a == item)
		i += 1
	end
	return -1
end

# convert array of names to JSON array
def namesJson( names )

	ret = '[ '
	start = true
	names.each do |n|
		unless start
			ret += ', '
		else
			start = false
		end
		ret += quote( n )	
	end
	ret += ' ]'
	return ret

end # namesJson()

# convert name (which may contain embedded quotes) to name in quotes
def quote( name )
	name.gsub!( '"', '\"' )
	name.gsub!( "'", "\'" )
	#	name.gsub!( "\", "\\\\" ) # how do you do this in Ruby?
	return "'" + name + "'"
end

puts
puts 'In main, calling launchWebDialog()'
launchWebDialog()

# end of vismap.rb